NOTIFICATION_TYPE_KICKOUT = 3
wus.threshold.time = 1/24/2   # Half an hour
acts.threshold = function(planned, done)
   # Give the same half-hour window except when the planned
   # duration is very short.
   ifelse(planned == 15, done >= planned,       # No window
   ifelse(planned == 30, done >= planned - 15,  # 15-minute window
   ifelse(planned == 45, done >= planned - 15,  # 15-minute window
                         done >= planned - 30))) # Half-hour window

score.cfc = function(v)
# This is my own guess, particularly with regard to which items
# are reverse-scored. Strathman, Gleicher, Boninger, and Edwards
# (1994) give the questionnaire but not scoring instructions.
    sum(ifelse(c(1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0), v, 6 - v))

rbl = function(seq, f) do.call(rbind, lapply(seq, f))

supplement = function(x, y)
     cbind(x, y[match(row.names(x), y[,1]), -1, drop = F])

format.s = function(s)
    sprintf("s%03d", int(s))

posix2times = function(posix, tz = "UTC")
   {lt = as.POSIXlt(posix.ct(posix), tz = tz)
    times((60 * (60 * lt$hour + lt$min) + lt$sec) / (60*60*24))}

db = dbConnect(dbDriver("SQLite"), dbname = "server.sqlite")

sn2s = dbGetQuery(db, "select sn, subject_id from Subjects")
sn2s = with(sn2s, `names<-`(format.s(subject_id), sn))

json = fromJSON("in-lab.json", null = NA)
json = json[order(int(sapply(json, function(x) x$subject)))]

econ = rbl(json, function(j)
    rbl(econ.tests, function(test)
        with(j$econ[[test]], data.frame(
            s = format.s(j$subject),
            test,
            tn = seq_along(small),
            small,
            big,
            catchtype,
            took_big,
            caught = ifelse(is.na(catchtype), NA,
               ifelse(catchtype == "big", !took_big,
               took_big))))))

wakeups = merge(all.x = T,
    rbl(json, function(j) data.frame(
        s = format.s(j$subject),
        date = as.Date(j$commitments$dates),
        planned = times(j$commitments$wakeups / (24*60)))),
    transform(dbReadTable(db, "WakeupTimes"),
        s = sn2s[char(sn)],
        date = as.Date(char(d8), "%Y%m%d"),
        submitted = posix2times(tz = "America/New_York", submitted_t),
        sn = NULL, d8 = NULL, submitted_t = NULL))

notes = rbl(json, function(j) data.frame(stringsAsFactors = F,
    s = format.s(j$subject),
    date = as.Date(j$commitments$dates),
    note = j$commitments$notes))

activity.names = rbl(json, function(j)
    rbl(seq_along(j$commitments$activities), function(an) data.frame(
        s = format.s(j$subject),
        an,
        activity = j$commitments$activities[[an]]$name)))

acodes = transform(read.delim("activity-codes.data", stringsAsFactors = F),
    acode = modlevels(factor(acode),
        virtue = "g", vice = "b", .NA = "u"))
uncoded = setdiff(unique(char(activity.names$activity)), acodes$activity)
if (length(uncoded))
    stop("Uncoded activites: ", paste0(col = ", ", uncoded))
activity.names = ordf(
    merge(activity.names, acodes)[qw(s, an, activity, acode)],
    s, an)

# There's an asymmetry with respect to dates and activities,
# which arises from how subjects report on their activities the
# day after the fact. This means:
# (1) We ask for reports about a day we don't ask for commitments
#     about: the day the subject is in lab.
# (2) We ask for commitments about a day we don't ask for
#     reports about: the last day the subject submits a report.
# Both cases should be excluded from analysis.
activities = merge(all.x = T,
  # Since all.y is FALSE, rows for (1) are omitted.
    rbl(json, function(j)
        rbl(seq_along(j$commitments$activities), function(an) data.frame(
            s = format.s(j$subject),
            an,
            date = as.Date(j$commitments$dates),
            planned = j$commitments$activities[[an]]$times))),
    transform(dbReadTable(db, "ActivityDurations"),
        s = sn2s[char(sn)],
        date = as.Date(char(d8), "%Y%m%d"),
        an = actn + 1,
        done = minutes, 
        sn = NULL, d8 = NULL, actn = NULL, minutes = NULL))
activities = ordf(activities, s, date, an)
# Manually remove rows for (2). Otherwise we could end up counting
# these commitments, which we never asked for reports about, as
# unfulfilled.
activities = ddply(activities, .(s), function(slice)
    ss(slice, date != max(date)))

ses = format.s(sapply(json, function(j) j$subject))
sb = data.frame(
    row.names = ses,
    task = sapply(json, function(j) j$task),
    started = as.POSIXct(tz = "UTC", sapply(json, function(j) j$overall_timing$started)),
    activs = ifna(
         with(activity.names, tapply(activity, s, length))[ses],
         0),
    plannedwu = table(ss(wakeups, !is.na(planned) & date < as.Date(Sys.time()))$s)[ses],
    caught = num(
        with(ss(econ, !is.na(catchtype)), tapply(caught, s, sum))[ses]),
    cfc = sapply(json, function(j) score.cfc(j$cfc)),
    con = sapply(json, function(j)
      {v = j$bfi
       R = function(x) 6 - x
       sum(c(v[3], R(v[8]), v[13], R(v[18]), R(v[23]),
           v[28], v[33], v[38], R(v[43])))}))
         # I'm using the sum, instead of the mean as specified
         # by the BFI authors, to make all the scores integers.
sb = supplement(sb, with(
    dbGetQuery(db, "select sn, d8 from Notifications where notification_type = ?",
        NOTIFICATION_TYPE_KICKOUT),
    data.frame(s = sn2s[char(sn)], kicked = as.Date(char(d8), "%Y%m%d"))))

## Remove from 'wakeups' and 'activities' rows for subjects and
## days during which the subject couldn'tve checked in because
## they'd been kicked out earlier.
#wakeups = ss(wakeups, !(s %in% names(kickouts) & date > kickouts[char(s)]))
#activities = ss(activities, !(s %in% names(kickouts) & date > kickouts[char(s)]))

# Compute self-control scores (scs), saving various intermediate
# values.
sb = supplement(sb, ddply(
    ss(wakeups,
        date < as.Date(Sys.time())),
    .(s),
    function(slice) data.frame(
        checkin.n = nrow(slice),
        checkin.s = sum(!is.na(slice$submitted)))))
sb = supplement(sb, ddply(
    ss(wakeups,
        date < as.Date(Sys.time()) &
        !is.na(planned) & !is.na(submitted)),
    .(s),
    function(slice) data.frame(
        wakeup.n = nrow(slice),
        wakeup.s = sum(slice$submitted - slice$planned < wus.threshold.time))))
sb = supplement(sb, ddply(
    ss(merge(activities, activity.names),
        date < as.Date(Sys.time()) &
        !is.na(planned) & !is.na(done) &
        acode == "virtue"),
    .(s),
    function(slice) data.frame(
        virtue.n = nrow(slice),
        virtue.s = sum(slice$done >= slice$planned))))
for (v in qw(checkin.n, checkin.s, wakeup.n, wakeup.s, virtue.n, virtue.s))
    sb[, v] = ifna(sb[, v], 0)
sb = transform(transform(sb,
    sc.n = checkin.n + wakeup.n + virtue.n,
    sc.s = checkin.s + wakeup.s + virtue.s),
    sc.p = sc.s / sc.n,
    # Here we use the formula for the posterior mean of a Bernoulli
    # distribution with a uniform prior.
    wus = (wakeup.s + 1) / (wakeup.n + 2),
    scs = (sc.s + 1) / (sc.n + 2))

bad.s = qw(
#    s001,
      # No commitments (no wakeup times and no activities)
    s006,
      # Caught 6 times
    s010)
      # Caught 11 times

sb$good = !(row.names(sb) %in% bad.s)
econg = ss(econ, !(s %in% bad.s))
